import numpy as np
from sklearn import metrics
from sklearn.svm import SVC
from sklearn.model_selection import train_test_split, GridSearchCV
from sklearn.preprocessing import scale
from scipy.io import loadmat

caltech101 = loadmat('caltech101_silhouettes_28.mat')

X = caltech101['X']
Y = caltech101['Y']

Y = Y.flatten()
unique, counts = np.unique(Y, return_counts=True)
classes = np.isin(Y, unique[(counts > 80) & (unique != 2)])

data = X[classes]
target = Y[classes]
images = np.reshape(data, (len(target), 28, 28))

data = scale(data.astype(float))

X_train, X_test, y_train, y_test, images_train, images_test = train_test_split(data, target, images, test_size=0.25, random_state=42)

# Create the SVM model with an RBF kernel
svm = SVC(kernel='rbf', random_state=42)

# Define the parameter grid for GridSearchCV
param_grid = {'C': [0.1, 1, 10, 100],
              'gamma': [0.001, 0.01, 0.1, 1]}

# Perform the grid search with cross-validation
grid_search = GridSearchCV(svm, param_grid, cv=5)
grid_search.fit(X_train, y_train)

# Get the best parameters
best_params = grid_search.best_params_
print("Best parameters: ", best_params)

# Perform the training using the best parameters
svm_best = SVC(kernel='rbf', random_state=42, **best_params)
svm_best.fit(X_train, y_train)

# Start evaluating
print("Training set score: %f" % svm_best.score(X_train, y_train))
print("Test set score: %f" % svm_best.score(X_test, y_test))
